#include "RepoEditorView.h"
#include "RepoEditorModel.h"
#include <QtGui/QHeaderView>
#include <QtCore/QObject>

RepoEditor::View::View(Model*    model,
                       QWidget*  parent)
   : HTableView(QStyleOptionViewItem::Bottom, 1, INT_MAX, parent),
     model(model)
{
   sortedModel = new QSortFilterProxyModel(parent);
   sortedModel->setSourceModel(model);
   setModel(sortedModel);

   setShowGrid(false);
   setGridStyle(Qt::NoPen);
   setAlternatingRowColors(true);
   setTabKeyNavigation(false);

   verticalHeader()->hide();
   setHorizontalHeader(new QHeaderView(Qt::Horizontal, this));

   horizontalHeader()->setSelectionBehavior(QAbstractItemView::SelectColumns);
   horizontalHeader()->setStretchLastSection(true);
   horizontalHeader()->setMovable(true);
   horizontalHeader()->setClickable(true);

   connect(horizontalHeader(), SIGNAL(sectionClicked(int)), 
           this, SLOT(handleHorizontalHeaderClicked(int)));

   connect(this, SIGNAL(clicked(const QModelIndex&)),
           this, SLOT(handleActivated(const QModelIndex&)));

   resizeColumnsToContents();
}

void  RepoEditor::View::handleHorizontalHeaderClicked(int  column)
{
   sortByColumn(column);
   sortedModel->setFilterKeyColumn(column);
}

void  RepoEditor::View::handleActivated(const QModelIndex&  ix)
{
   if (ix.column() == USE) {
      model->toggleUse(sortedModel->mapToSource(ix));
   }
}

int  RepoEditor::View::currentRow()
{
   return sortedModel->mapToSource(currentIndex()).row();
}

void  RepoEditor::View::setCurrentRow (int  row)
{
   setCurrentIndex(sortedModel->mapFromSource(model->createIndex(row)));
}

QModelIndexList  RepoEditor::View::selectedRows() const
{
   QModelIndexList  result = selectedIndexes();
   QSet<int>        seen;

   QModelIndexList::iterator  it = result.begin();
   while (it != result.end()) {
      if (seen.contains(it->row())) {
         it = result.erase(it);
      } else {
         *it = sortedModel->mapToSource(*it);
         seen.insert(it->row());
         ++it;
      }
   }

   return result;
}

void  RepoEditor::View::currentChanged(const QModelIndex&  current,
                                       const QModelIndex&  previous)
{
   int  currentRow  = sortedModel->mapToSource(current).row();
   int  previousRow = sortedModel->mapToSource(previous).row();
   if (currentRow != previousRow) {
      emit currentChanged(currentRow, previousRow);
   }
}
